/* BearPaw1200CU panel

   Copyright (C) 2001 Mustek. 
   Maintained by Tom Wang and Alex Lin
   <tom.wang@mustek.com.tw, alex_lin@mustek.com.tw>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
*/


/**
 * ImageWindow.c
 * 
 * All the code to work with an image in a window 
 */

#include <gnome.h>
#include "imageviewerutility.h"

struct WndRegion
{
    double xy;
    double wh;
};


/** 
 * DestroyWindow () 
 * 
 * Callback triggered when the image window is destroyed. 
 * A pointer to the image on display is passed into the handler 
 * in order that the image can be killed, preventing a memory 
 * leak fom occuring. The Active window pointer is also cleared. 
 */
void DestroyWindow ( GtkWidget * window, gpointer imagedata )
{
   gdk_imlib_kill_image ( ( GdkImlibImage * ) imagedata );
   ClearActiveWindow ();
}

/**
 * GotFocus ()
 * 
 * Callback triggered when image window gets focus. It then 
 * stores a pointer to itself and its image as the active window 
 * and image, in order that main window gui can work with the 
 * correct window 
 */
void GotFocus ( GtkWidget * window, GdkEvent * event, gpointer imagedata )
{
   SetActiveWindow ( window, ( GdkImlibImage * ) imagedata );
}

/**
 * SetImageIntoWindow ()
 * 
 * A utility function that takes a GdkImLibImage and puts it into 
 * a specified window 
 */
void SetImageIntoWindow ( GtkWidget * window, GdkImlibImage * image )
{
   GtkWidget *pixmap;

   /* Check to see if there is already a child in the window */
   if (GTK_BIN (window)->child != NULL)
     /* Destroy the child if there is one */
      gtk_widget_destroy ( GTK_BIN ( window )->child );
   
   /* Create a new GnomePixmap from the passed in image */
   pixmap = gnome_pixmap_new_from_imlib ( image );
   
   /* Add the GnomePixmap into the window and show it */
   gtk_container_add ( GTK_CONTAINER ( window ), pixmap );
   gtk_widget_show ( pixmap );
}


/* callback to change the zoom factor of the canvas in response
 * to the spinbutton widget 
 */
void ChangeZoomFactor ( GtkEditable * spin, gpointer data )
{
   GtkWidget * canvas = GTK_WIDGET ( data );

   gnome_canvas_set_pixels_per_unit ( GNOME_CANVAS ( canvas ), 
	      ( double ) gtk_spin_button_get_value_as_float ( GTK_SPIN_BUTTON ( spin ) ) );

   return ;
}

/**
 * GetWndRegion()
 *
 * Window region is depend on image size 
 */
struct WndRegion GetWndRegion( GdkImlibImage * image )
{
    struct WndRegion wndregion;
  
    if( image->rgb_width >= 801 && image->rgb_width <= 1200 ) {
        wndregion.xy = 4.0;
        wndregion.wh = 2.0;
    }
    else if(  image->rgb_width >= 1201 ) {
        wndregion.xy = 10.0;
	wndregion.wh = 5.0;
    }
    else {
        wndregion.xy = 2.0;
        wndregion.wh = 1.0;
    }

    return wndregion;
}
    
/**
 * CreateCanvasItem ()
 * 
 * A utility function that takes a GdkImLibImage and puts it into 
 * a specified canvas 
 */
void CreateCanvasItem ( GtkWidget * window, GtkWidget * canvas, GdkImlibImage * image )
{
  
   GnomeCanvasGroup * group;
   GnomeCanvasItem * widget;
   struct WndRegion wndregion;
   
   /* Check to see if there is already a child in the window */
   if (GTK_BIN (window)->child != NULL)
     /* Destroy the child if there is one */
     gtk_widget_destroy ( GTK_BIN ( window )->child );
  
    
   /* Retrieve the canvas root group */
   group = gnome_canvas_root ( GNOME_CANVAS ( canvas ) );


   /* Window region is depend on image size */
   wndregion = GetWndRegion( image );
   widget = gnome_canvas_item_new (
	    group, gnome_canvas_image_get_type(),
	    "image", image,
	    "x", ((double)image->rgb_width) / wndregion.xy,
	    "y", ((double)image->rgb_height) / wndregion.xy,
	    "anchor", GTK_ANCHOR_CENTER,
	    "width", ((double)image->rgb_width) / wndregion.wh,
	    "height",((double)image->rgb_height)  / wndregion.wh,
	    NULL
	    );

   /* Mark the area of canvas we want visible */
   gnome_canvas_set_scroll_region ( GNOME_CANVAS ( canvas ), 0.0, 0.0,
				    image->rgb_width / wndregion.wh,
				    image->rgb_height / wndregion.wh );
   
   gtk_window_set_default_size( GTK_WINDOW ( window ),
				(image->rgb_width) / wndregion.wh + 0,
				(image->rgb_height) / wndregion.wh + 0);
   
}


/** 
 * FlipImage ()
 * 
 * Utility function to flip an image about an axis. The second
 * parameter dictates whether the calling code wants the image
 * to be flipped horizontally, or vertically
 */
void FlipImage ( gboolean horizontal )
{
   GtkWidget *window;
   GdkImlibImage *image;

   /* Grab the active window pointer */
   window = GetActiveWindow ();

   /* Check the pointer isn't null before continuing */
   if ( window != NULL )
   {
      /* grab the image and perform the flip */
      image = GetActiveImage ();
      if ( horizontal )
         gdk_imlib_flip_image_horizontal ( image );
      else
         gdk_imlib_flip_image_vertical ( image );

      /* Set the flipped image into the active window */
      SetImageIntoWindow ( window, image );
   }
}

/** 
 * RotateImage ()
 *
 * Utility to rotate an image 90 degrees. 
 */
void RotateImage ()
{
   GtkWidget *window;
   GdkImlibImage *image;
   struct WndRegion wndregion;

   /* Grab the active window */
   window = GetActiveWindow ();

   /*  Check that the active window is valid before continuing */
   if ( window != NULL )
   {
      /* Grab the active image */
      image = GetActiveImage ();

      /* Rotate the image - second parameter is ignored at the moment */
      gdk_imlib_rotate_image ( image, 0 );

      /* Set the new image into the window */
      SetImageIntoWindow ( window, image );

      /* Window region is depend on image size */
      wndregion = GetWndRegion( image );   
      gtk_window_set_default_size( GTK_WINDOW ( window ),
				   (image->rgb_width) / wndregion.wh + 10,
				   (image->rgb_height) / wndregion.wh + 10);
   }
}

/** 
 * CloseActiveImage () 
 * 
 * Utility function to close down the last active window. 
 */
void CloseActiveImage ()
{
   /* Grab the last active window  pointer */
   GtkWidget *window = GetActiveWindow ();

   /* If the pointer is valid, destroy the window */
   if (window != NULL)
      gtk_widget_destroy (window);
}

/** 
 * ShowImage ()
 * 
 * The main entry function for the image window. 
 * It takes the passed in filename and attempts to 
 * load it up as an image 
 **/
void ShowImage (gchar * filename)
{
  
   GdkImlibImage * image;
   GtkWidget * window, * messagebox;
   GtkWidget * canvas;
   GtkWidget * scrollwindow;

   

   /* Attempt to load up the specified image */
   image = gdk_imlib_load_image (filename);

   /* If the resulting image is NULL, then the load failed....
    * tell the user */
   if(image == NULL)
   {
      messagebox = gnome_message_box_new( "Sorry, you specified an invalid file",
					  GNOME_MESSAGE_BOX_ERROR,
					  GNOME_STOCK_BUTTON_OK, NULL );
      gnome_dialog_run_and_close ( GNOME_DIALOG ( messagebox ) );
      
      return;
   }

   /* Create a window in the usual way, with the filename as the title */
   window = gtk_window_new ( GTK_WINDOW_TOPLEVEL );
   gtk_window_set_default_size( GTK_WINDOW ( window ), 200, 200 );
   gtk_window_set_title ( GTK_WINDOW ( window ), filename );


   /* ================================ Canvas ================================ */

   /* Set up the visual and colormap */
   gtk_widget_push_visual ( gdk_imlib_get_visual () );
   gtk_widget_push_colormap ( gdk_imlib_get_colormap () );

   /* Create the canvas in Gdk Mode */
   canvas = gnome_canvas_new ();

   /* Now pop the visual and colormap */
   gtk_widget_pop_colormap ();
   gtk_widget_pop_visual ();


   //SetImageIntoWindow ( window, image );
   CreateCanvasItem ( window, canvas, image );


   /* ========================== Scroll Window ================================ */
   
   /* Pass the adjustment objects across to the scrolled window */
   scrollwindow = gtk_scrolled_window_new ( NULL, NULL);
   gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(scrollwindow), 
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   /* Since the gtkList can't scroll on its own, ask the scrolled window to create \
      display the canvas inside a GtkViewport */
   //   gtk_widget_set_usize( scrollwindow, 200, 200 );
   gtk_container_add(GTK_CONTAINER(scrollwindow), canvas );

   /*****************************************************************************/
   
   /* Connect up the window's signal handlers */
   gtk_signal_connect ( GTK_OBJECT ( window ), "destroy",
			GTK_SIGNAL_FUNC ( DestroyWindow ), image );

   gtk_signal_connect ( GTK_OBJECT ( window ), "focus_in_event",
			GTK_SIGNAL_FUNC (GotFocus), image );

   gtk_container_add ( GTK_CONTAINER ( window ), scrollwindow );
   
   /* Finally, show the window */
   gtk_widget_show_all ( window );

}









