/* -*- linux-c -*- */

/* 
 * Driver for GT-6801 USB Scanners (linux-2.4.0)
 *
 * Copyright (C) 2001 Anthony Sale
 * anthony_sale@jumble.i12.com
 *
 * 
 * Based on scanner.c by David E. Nelson
 *
 * Portions may be copyright David E. Nelson, Brad Keryan and Michael Gee.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Originally based upon mouse.c (Brad Keryan) and printer.c (Michael Gee).
 *
 * History
 *
 *  0.1  	19/07/2001
 *		Development started on kernel 2.4.05
 */
 
/* 
 * Definitions, macros, module info, 
 * debug/ioctl/data_dump enable, and other constants.
 */ 
#include "gt6801.h"


static int
open_gt6801(struct inode * inode, struct file * file)
{
	struct gt6801_usb_data *gt6801;
	struct usb_device *dev;

	kdev_t gt6801_minor;

	int err=0;

	lock_kernel();

	gt6801_minor = USB_GT6801_MINOR(inode);

	dbg("open_gt6801: gt6801_minor:%d", gt6801_minor);

	if (!p_gt6801_table[gt6801_minor]) {
		err("open_gt6801(%d): Unable to access minor data", gt6801_minor);
		err = -ENODEV;
		unlock_kernel();
		return err;
	}
	
	gt6801 = p_gt6801_table[gt6801_minor];

	dev = gt6801->gt6801_dev;

	if (!dev) {
		err("open_gt6801(%d): Scanner device not present", gt6801_minor);
		err = -ENODEV;
		unlock_kernel();
		return err;
	}

	if (!gt6801->present) {
		err("open_gt6801(%d): Scanner is not present", gt6801_minor);
		err = -ENODEV;
		unlock_kernel();
		return err;
	}

	if (gt6801->isopen) {
		err("open_gt6801(%d): Scanner device is already open", gt6801_minor);
		err = -EBUSY;
		unlock_kernel();
		return err;
	}

	init_waitqueue_head(&gt6801->rd_wait_q);

	gt6801->isopen = 1;

	file->private_data = gt6801; /* Used by the read and write metheds */

	MOD_INC_USE_COUNT;

	unlock_kernel();

	return err;
}

static int
close_gt6801(struct inode * inode, struct file * file)
{
	struct gt6801_usb_data *gt6801;

	kdev_t gt6801_minor;

	gt6801_minor = USB_GT6801_MINOR (inode);

	dbg("close_gt6801: gt6801_minor:%d", gt6801_minor);

	if (!p_gt6801_table[gt6801_minor]) {
		err("close_gt6801(%d): invalid gt6801_minor", gt6801_minor);
		return -ENODEV;
	}

	gt6801 = p_gt6801_table[gt6801_minor];

	gt6801->isopen = 0;

	file->private_data = NULL;

	MOD_DEC_USE_COUNT;

	return 0;
}


static ssize_t
read_gt6801(struct file * file, char * buffer,
             size_t count, loff_t *ppos)
{
	struct gt6801_usb_data *gt6801;
	struct usb_device *dev;

	ssize_t bytes_read;	/* Overall count of bytes_read */
	ssize_t ret;

	kdev_t gt6801_minor;

	int partial;		/* Number of bytes successfully read */
	int this_read;		/* Max number of bytes to read */
	int result;
	int rd_expire = RD_EXPIRE;

	char *ibuf;
	
	gt6801 = file->private_data;

	gt6801_minor = gt6801->gt6801_minor;
	
	dbg("read entry(%d): count:%d", gt6801_minor, count); 

	ibuf = gt6801->ibuf;
	memset(ibuf, 0, IBUF_SIZE);

	dev = gt6801->gt6801_dev;

	bytes_read = 0;
	ret = 0;

	file->f_dentry->d_inode->i_atime = CURRENT_TIME; /* Update the
                                                            atime of
                                                            the device
                                                            node */
	down(&(gt6801->gen_lock));

	while (count > 0) {
		if (signal_pending(current)) {
			ret = -EINTR;
			break;
		}

		this_read = (count >= IBUF_SIZE) ? IBUF_SIZE : count;

		result = usb_bulk_msg(dev, usb_rcvbulkpipe(dev, gt6801->bulk_in_ep), ibuf, this_read, &partial, gt6801->rd_nak_timeout);
		dbg("read stats(%d): result:%d this_read:%d partial:%d count:%d", gt6801_minor, result, this_read, partial, count); 

/*
 * Scanners are sometimes inheriently slow since they are mechanical
 * in nature.  USB bulk reads tend to timeout while the scanner is
 * positioning, resetting, warming up the lamp, etc if the timeout is
 * set too low.  A very long timeout parameter for bulk reads was used
 * to overcome this limitation, but this sometimes resulted in folks
 * having to wait for the timeout to expire after pressing Ctrl-C from
 * an application. The user was sometimes left with the impression
 * that something had hung or crashed when in fact the USB read was
 * just waiting on data.  So, the below code retains the same long
 * timeout period, but splits it up into smaller parts so that
 * Ctrl-C's are acted upon in a reasonable amount of time.
 */

		if (result == -ETIMEDOUT) { /* NAK */
			if (!partial) { /* No data */
				if (--rd_expire <= 0) {	/* Give it up */
					warn("read_gt6801(%d): excessive NAK's received", gt6801_minor);
					ret = result;
					break;
				} else { /* Keep trying to read data */
					interruptible_sleep_on_timeout(&gt6801->rd_wait_q, gt6801->rd_nak_timeout);
					continue;
				}
			} else { /* Timeout w/ some data */
				goto data_recvd;
			}
		}
		
		if (result == -EPIPE) { /* No hope */
			if(usb_clear_halt(dev, gt6801->bulk_in_ep)) {
				err("read_gt6801(%d): Failure to clear endpoint halt condition (%Zd).", gt6801_minor, ret);
			}
			ret = result;
			break;
		} else if ((result < 0) && (result != USB_ST_DATAUNDERRUN)) {
			warn("read_gt6801(%d): funky result:%d. Please notify the maintainer.", gt6801_minor, (int)result);
			ret = -EIO;
			break;
		}

	data_recvd:

		if (partial) { /* Data returned */
			if (copy_to_user(buffer, ibuf, partial)) {
				ret = -EFAULT;
				break;
			}
			count -= this_read; /* Compensate for short reads */
			bytes_read += partial; /* Keep tally of what actually was read */
			buffer += partial;
		} else {
			ret = 0;
			break;
		}
	}
	up(&(gt6801->gen_lock));

	return ret ? ret : bytes_read;
}

static int 
ioctl_gt6801(struct inode *inode, struct file *file, 
		unsigned int cmd, unsigned long arg)
{
	struct usb_device *dev;
	
	int result;
	
	int pipe = 0;

	kdev_t gt6801_minor;
	
	void *buffer= NULL;
	char *cbuffer = NULL;
	
	struct {
		__u8 request;
		__u8 requesttype;
		__u16 value;
		__u16 index;
		__u16 size;
		void *data;
		} args;
		
	gt6801_minor = USB_GT6801_MINOR(inode);
	
	if (!p_gt6801_table[gt6801_minor]) {
		err("ioctl_gt6801(%d): invalid gt6801_minor", gt6801_minor);
		return -ENODEV;
	}
	
	dev = p_gt6801_table[gt6801_minor]->gt6801_dev;
		
	if(cmd != GT6801_IOCTL)
	{
		return -ENOIOCTLCMD;
	}
	else
	{
		//get args from user
		if(copy_from_user(&args, (void *)arg, sizeof(args)))
			return -EFAULT;
			
		//build data buffer and copy data from user if required
		if(args.size)
		{
			if(!(buffer = kmalloc(args.size, GFP_KERNEL))) {
				err("ioctl_gt6801: Out of memory");
				return -EFAULT;
			}
			if(copy_from_user(buffer, args.data, args.size))
				return -EFAULT;
			
			cbuffer = buffer;
		}
		
		if((args.requesttype & USB_DIR_IN) == USB_DIR_IN)
			pipe = usb_rcvctrlpipe(dev,0);
		else if((args.requesttype & USB_DIR_OUT) == USB_DIR_OUT)
			pipe = usb_sndctrlpipe(dev,0);
		else
		{
			dbg("ioctl_gt6801: requesttype %x is invalid",
				args.requesttype);
			return -EFAULT;
		}
		
		dbg("ioctl_gt6801: before buffer[0] = %d, buffer[1] = %d", cbuffer[0], cbuffer[1]);
		result = usb_control_msg(dev, pipe,
						args.request, args.requesttype,
						args.value, args.index,
						buffer, args.size, HZ*5); 

		dbg("ioctl_gt6801: after buffer[0] = %d, buffer[1] = %d", cbuffer[0], cbuffer[1]);
		dbg("ioctl_gt6801: result was %d", result);
		
		//copy data back to user				
		if(args.size)
		{
			if(copy_to_user(args.data, buffer, args.size))
			{
				kfree(buffer);
				return -EFAULT;
			}
			kfree(buffer);
		}
		
		if(copy_to_user((void *)arg, &args, sizeof(args)))
			return -EFAULT;
	}
	return result;
}

static void *
probe_gt6801(struct usb_device *dev, unsigned int ifnum,
	      const struct usb_device_id *id)
{
	struct gt6801_usb_data *gt6801;
	struct usb_interface_descriptor *interface;
	struct usb_endpoint_descriptor *endpoint;

	int ep_cnt;
	int ix;

	kdev_t gt6801_minor;

	char valid_device = 0;
	char have_bulk_in;

	if (vendor != -1 && product != -1) {
		info("probe_gt6801: User specified USB scanner -- Vendor:Product - %x:%x", vendor, product);
	}

	dbg("probe_gt6801: USB dev address:%p", dev);
	dbg("probe_gt6801: ifnum:%u", ifnum);

/*
 * 1. Check Vendor/Product
 * 2. Determine/Assign Bulk Endpoint
 */

/*
 * The GT-6801 scanner on a chip only has one end point 
 */

	for (ix = 0; ix < sizeof (gt6801_device_ids) / sizeof (struct usb_device_id); ix++) {
		if ((dev->descriptor.idVendor == gt6801_device_ids [ix].idVendor) &&
		    (dev->descriptor.idProduct == gt6801_device_ids [ix].idProduct)) {
			valid_device = 1;
			break;
                }
	}
	if (dev->descriptor.idVendor == vendor &&   /* User specified */
	    dev->descriptor.idProduct == product) { /* User specified */
		valid_device = 1;
	}

        if (!valid_device)
                return NULL;    /* We didn't find anything pleasing */

/*
 * We have found a vendor & prodcut we like
 */

	if (dev->descriptor.bNumConfigurations != 1) {
		info("probe_gt6801: Only one device configuration is supported.");
		return NULL;
	}

	if (dev->config[0].bNumInterfaces != 1) {
		info("probe_gt6801: Only one device interface is supported.");
		return NULL;
	}

	interface = dev->config[0].interface[ifnum].altsetting;	
	endpoint = interface[ifnum].endpoint;

/*
 * GT-6801 only has one endpoint
 */

	dbg("probe_gt6801: Number of Endpoints:%d", (int) interface->bNumEndpoints);

	if (interface->bNumEndpoints != 1) {
		info("probe_gt6801: Only one endpoint supported.");
		return NULL;
	}

	ep_cnt = have_bulk_in = 0;

	while (ep_cnt < interface->bNumEndpoints) {

		if (!have_bulk_in && IS_EP_BULK_IN(endpoint[ep_cnt])) {
			ep_cnt++;
			have_bulk_in = ep_cnt;
			dbg("probe_gt6801: bulk_in_ep:%d", have_bulk_in);
			continue;
		}

		info("probe_gt6801: Undetected endpoint. Notify the maintainer.");
		return NULL;	/* Shouldn't ever get here unless we have something weird */
	}


/*
 * Perform a quick check to make sure that everything worked as it
 * should have.
 */

	switch(interface->bNumEndpoints) {
	case 1:
		if (have_bulk_in)
		{
			info("probe_gt6801: Found bulk in endpoint.");
		}
		break;
	default:
		info("probe_gt6801: Endpoint determination failed.  Notify the maintainer.");
		return NULL;
	}


/*
 * Determine a minor number and initialize the structure associated
 * with it.  The problem with this is that we are counting on the fact
 * that the user will sequentially add device nodes for the scanner
 * devices.  */

	for (gt6801_minor = 0; gt6801_minor < GT6801_MAX_MNR; gt6801_minor++) {
		if (!p_gt6801_table[gt6801_minor])
			break;
	}

/* Check to make sure that the last slot isn't already taken */
	if (p_gt6801_table[gt6801_minor]) {
		err("probe_gt6801: No more minor devices remaining.");
		return NULL;
	}

	dbg("probe_gt6801: Allocated minor:%d", gt6801_minor);

	if (!(gt6801 = kmalloc (sizeof (struct gt6801_usb_data), GFP_KERNEL))) {
		err("probe_gt6801: Out of memory.");
		return NULL;
	}
	memset (gt6801, 0, sizeof(struct gt6801_usb_data));
	dbg ("probe_gt6801(%d): Address of gt6801:%p", gt6801_minor, gt6801);


	/* Ok, now initialize all the relevant values */
	if (!(gt6801->obuf = (char *)kmalloc(OBUF_SIZE, GFP_KERNEL))) {
		err("probe_gt6801(%d): Not enough memory for the output buffer.", gt6801_minor);
		kfree(gt6801);
		return NULL;
	}
	dbg("probe_gt6801(%d): obuf address:%p", gt6801_minor, gt6801->obuf);

	if (!(gt6801->ibuf = (char *)kmalloc(IBUF_SIZE, GFP_KERNEL))) {
		err("probe_gt6801(%d): Not enough memory for the input buffer.", gt6801_minor);
		kfree(gt6801->obuf);
		kfree(gt6801);
		return NULL;
	}
	dbg("probe_gt6801(%d): ibuf address:%p", gt6801_minor, gt6801->ibuf);
	

	gt6801->rd_nak_timeout = RD_NAK_TIMEOUT;
	

	if (read_timeout > 0) {	/* User specified read timeout overrides everything */
		info("probe_gt6801: User specified USB read timeout - %d", read_timeout);
		gt6801->rd_nak_timeout = read_timeout;
	}


	gt6801->bulk_in_ep = have_bulk_in;
	gt6801->present = 1;
	gt6801->gt6801_dev = dev;
	gt6801->gt6801_minor = gt6801_minor;
	gt6801->isopen = 0;

	init_MUTEX(&(gt6801->gen_lock));

	return p_gt6801_table[gt6801_minor] = gt6801;
}

static void
disconnect_gt6801(struct usb_device *dev, void *ptr)
{
	struct gt6801_usb_data *gt6801 = (struct gt6801_usb_data *) ptr;

	usb_driver_release_interface(&gt6801_driver,
                &gt6801->gt6801_dev->actconfig->interface[gt6801->ifnum]);

	kfree(gt6801->ibuf);
	kfree(gt6801->obuf);

	dbg("disconnect_gt6801: De-allocating minor:%d", gt6801->gt6801_minor);
	p_gt6801_table[gt6801->gt6801_minor] = NULL;
	kfree (gt6801);
}

static struct
file_operations usb_gt6801_fops = {
	read:		read_gt6801,
	//write:		write_gt6801,
	ioctl:          ioctl_gt6801,
	open:		open_gt6801,
	release:	close_gt6801,
};

static struct
usb_driver gt6801_driver = {
	name:		"usb-gt6801",
	probe:		probe_gt6801,
	disconnect:	disconnect_gt6801,
	fops:		&usb_gt6801_fops,
	minor:		GT6801_BASE_MNR,
	id_table:	NULL, /* This would be scanner_device_ids, but we
				 need to check every USB device, in case
				 we match a user defined vendor/product ID. */
};

void __exit
usb_gt6801_exit(void)
{
	usb_deregister(&gt6801_driver);
}

int __init
usb_gt6801_init (void)
{
        if (usb_register(&gt6801_driver) < 0)
                return -1;

	info("GT-6801 USB Scanner support registered.");
	return 0;
}

module_init(usb_gt6801_init);
module_exit(usb_gt6801_exit);
